#include "col_test.h"

DisplayDevice *pDisplay; // Pointer to Display
Rasterizer *pRaster;     // Rasterizer to Internal Screen
Surface *pScreen;        // Internal Screen
Col_Test *pCol_Test;     // Game Pointer
CComModule _Module;
int fps, currentFrame, screenWidth, screenHeight;
static DWORD frameTicks[16];

Pixel cTransparent = Color(238, 0, 151);

// Constructor
Col_Test::Col_Test()
{
  m_config.szWindowName     = _T(GAME_NAME);
  m_config.orientation      = ORIENTATION_NORTH;
  m_config.splashScreenTime = 0;
  m_config.frameRate        = 0;
}

// Destructor
Col_Test::~Col_Test()
{
}

// Game Initialization
bool Col_Test::GameInit()
{
  pDisplay     = GetDisplay();
  screenWidth  = 240; // GetDisplay()->GetWidth();
  screenHeight = 320; // GetDisplay()->GetHeight();
  bShutdown    = false;

  asteroid = LoadImage(pDisplay, IDR_ASTEROID);
  asteroid->SetColorMask( cTransparent );
  ship = LoadImage(pDisplay, IDR_SHIP);
  ship->SetColorMask( cTransparent );

  pt_asteroid.x = 80;
  pt_asteroid.y = 80;
  pt_ship.x = 120;
  pt_ship.y = 240;

  dx_asteroid = 1;
  dy_asteroid = -1;

  dx_ship = 0;
  dy_ship = 0;

  m_Font.load(pDisplay, IDR_FONT);
  m_Font.getSurface()->SetColorMask(Color(255, 0, 255));

  return true;
}

// Game Loop
void Col_Test::GameLoop()
{
  GetFrameRate();
  Update();
}

// Game Clean-Up
void Col_Test::GameEnd()
{
}

// Frame Update - Called 30 times per second (approximately)
void Col_Test::Update()
{
  pt_asteroid.x += dx_asteroid;
  pt_asteroid.y += dy_asteroid;
  pt_ship.x += dx_ship;
  pt_ship.y += dy_ship;

  if((pt_asteroid.x + asteroid->GetWidth() > screenWidth - 1) || (pt_asteroid.x < 0))
  {
    pt_asteroid.x -= dx_asteroid;
    dx_asteroid = 0 - dx_asteroid;
  }
  if((pt_asteroid.y + asteroid->GetHeight() > screenHeight - 1) || (pt_asteroid.y < 0))
  {
    pt_asteroid.y -= dy_asteroid;
    dy_asteroid = 0 - dy_asteroid;
  }

  if((pt_ship.x + ship->GetWidth() > screenWidth - 1) || (pt_ship.x < 0))
  {
    pt_ship.x -= dx_ship;
    dx_ship = 0 - dx_ship;
  }
  if((pt_ship.y + ship->GetHeight() > screenHeight - 1) || (pt_ship.y < 0))
  {
    pt_ship.y -= dy_ship;
    dy_ship = 0 - dy_ship;
  }

  pDisplay->Clear( 0x0 );
  pDisplay->Blit(pt_asteroid.x, pt_asteroid.y, asteroid);
  pDisplay->Blit(pt_ship.x, pt_ship.y, ship);

  m_Font.drawf(pDisplay, 1, 1, FONT_LEFT, TEXT("FPS: %d"), fps);
  if(Collision(asteroid, pt_asteroid, ship, pt_ship))
    m_Font.drawf(pDisplay, 238, 1, FONT_RIGHT, TEXT("Collision!"));

  if(bShutdown)
    Shutdown();
  else
    pDisplay->Update();
}

// Button Down - Called when a button is pressed.
void Col_Test::ButtonDown(int button)
{
  const InputDevice::KeyList& keys = GetInput()->GetKeyList();
  if(button == keys.vkUp)    dy_ship -= 1;
  if(button == keys.vkDown)  dy_ship += 1;
  if(button == keys.vkRight) dx_ship += 1;
  if(button == keys.vkLeft)  dx_ship -= 1;
  if(button == keys.vkA)     { dx_asteroid = 0; dy_asteroid = 0; }
  if(button == keys.vkB)     { dx_asteroid = -1; dy_asteroid = 1; }
}

// Button Up - Called when a button is released.
void Col_Test::ButtonUp(int button)
{
  // Not used in this game.
}

// Stylus Down
void Col_Test::StylusDown(Point stylus)
{
  // Not used in this game.
}

// Stylus Up
void Col_Test::StylusUp(Point stylus)
{
  // Not used in this game.
  bShutdown = true;
}

// Stylus Move
void Col_Test::StylusMove(Point stylus)
{
  // Not used in this game.
}

// WinMain - Application Entry Point
int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPTSTR, int )
{
  _Module.Init( 0, hInstance );
  pCol_Test = new Col_Test();
  pCol_Test->Run();
  return 0;
}

bool Collision(Surface *s1, Point p1, Surface *s2, Point p2)
{
  return Collision(s1, p1.x, p1.y, s2, p2.x, p2.y);
}

bool Collision(Surface *s1, int x1, int y1, Surface *s2, int x3, int y3)
{
  int x2 = x1 + s1->GetWidth()  - 1;
  int y2 = y1 + s1->GetHeight() - 1;
  int x4 = x3 + s2->GetWidth()  - 1;
  int y4 = y3 + s2->GetHeight() - 1;
  int x5, y5, x6, y6;

  Pixel c1 = s1->GetColorMask();
  Pixel c2 = s2->GetColorMask();
  
  Rasterizer *r1 = pDisplay->CreateRasterizer(s1);
  Rasterizer *r2 = pDisplay->CreateRasterizer(s2);

  // Rule out non-collisions immediately
  if((x1 > x4) || (x2 < x3) || (y1 > y4) || (y2 < y3))
    return false;

  // Determine the x,y points of the collision rectangle  
  x5 = (x1 > x3) ? x1 : x3;
  y5 = (y1 > y3) ? y1 : y3;
  x6 = (x2 < x4) ? x2 : x4;
  y6 = (y2 < y4) ? y2 : y4;

  // Test the pixels
  for(int x = x5; x <= x6; x++)
    for(int y = y5; y <= y6; y++)
      if((r1->GetPixel(x-x1, y-y1) != c1) && (r2->GetPixel(x-x3, y-y3) != c2))
        return true;

  return false;
}

void GetFrameRate()
{
  DWORD tick = ::GetTickCount();
  frameTicks[ currentFrame & 15 ] = tick;
  
  if (++currentFrame >= 16)
  {
    DWORD time = tick - frameTicks[ currentFrame & 15 ];
    
    if (time > 0)
      fps = 16000 / time;
    else
      fps = 30;
  }
}
